@@ -292,7 +292,7 @@ SyntheticClient::SyntheticClient(Client *client, int w)
 
 void *synthetic_client_thread_entry(void *ptr)
 {
-  SyntheticClient *sc = (SyntheticClient*)ptr;
+  SyntheticClient *sc = static_cast<SyntheticClient*>(ptr);
   //int r = 
   sc->run();
   return 0;//(void*)r;
@@ -340,7 +340,7 @@ int SyntheticClient::run()
 
   for (list<int>::iterator it = modes.begin();
        it != modes.end();
-       it++) {
+       ++it) {
     int mode = *it;
     dout(3) << "mode " << mode << dendl;
 
@@ -827,7 +827,7 @@ int SyntheticClient::run()
         int count = iargs.front();  iargs.pop_front();
         if (run_me()) {
           for (int i=0; i<count; i++) {
-            int fd = client->open("test", rand()%2 ? (O_WRONLY|O_CREAT):O_RDONLY);
+            int fd = client->open("test", (rand()%2) ? (O_WRONLY|O_CREAT) : O_RDONLY);
             if (fd > 0) client->close(fd);
           }
         }
@@ -1481,25 +1481,25 @@ int SyntheticClient::play_trace(Trace& t, string& prefix, bool metadata_only)
   // close open files
   for (hash_map<int64_t, int64_t>::iterator fi = open_files.begin();
        fi != open_files.end();
-       fi++) {
+       ++fi) {
     dout(1) << "leftover close " << fi->second << dendl;
     if (fi->second > 0) client->close(fi->second);
   }
   for (hash_map<int64_t, dir_result_t*>::iterator fi = open_dirs.begin();
        fi != open_dirs.end();
-       fi++) {
+       ++fi) {
     dout(1) << "leftover closedir " << fi->second << dendl;
     if (fi->second != 0) client->closedir(fi->second);
   }
   for (hash_map<int64_t,Fh*>::iterator fi = ll_files.begin();
        fi != ll_files.end();
-       fi++) {
+       ++fi) {
     dout(1) << "leftover ll_release " << fi->second << dendl;
     if (fi->second) client->ll_release(fi->second);
   }
   for (hash_map<int64_t,void*>::iterator fi = ll_dirs.begin();
        fi != ll_dirs.end();
-       fi++) {
+       ++fi) {
     dout(1) << "leftover ll_releasedir " << fi->second << dendl;
     if (fi->second) client->ll_releasedir(fi->second);
   }
@@ -1521,7 +1521,7 @@ int SyntheticClient::clean_dir(string& basedir)
 
   for (list<string>::iterator it = contents.begin();
        it != contents.end();
-       it++) {
+       ++it) {
     if (*it == ".") continue;
     if (*it == "..") continue;
     string file = basedir + "/" + *it;
@@ -1580,7 +1580,7 @@ int SyntheticClient::full_walk(string& basedir)
     
     for (list<string>::iterator it = contents.begin();
 	 it != contents.end();
-	 it++) {
+	 ++it) {
       if (*it == "." ||
 	  *it == "..") 
 	continue;
@@ -1638,7 +1638,7 @@ int SyntheticClient::full_walk(string& basedir)
     }
   }
 
-  for (hash_map<inodeno_t,int>::iterator p = nlink.begin(); p != nlink.end(); p++) {
+  for (hash_map<inodeno_t,int>::iterator p = nlink.begin(); p != nlink.end(); ++p) {
     if (nlink_seen[p->first] != p->second)
       dout(0) << p->first << " nlink " << p->second << " != " << nlink_seen[p->first] << "seen" << dendl;
   }
@@ -1977,7 +1977,10 @@ int SyntheticClient::write_file(string& fn, int size, loff_t wrsize)   // size i
 
   int fd = client->open(fn.c_str(), O_RDWR|O_CREAT);
   dout(5) << "writing to " << fn << " fd " << fd << dendl;
-  if (fd < 0) return fd;
+  if (fd < 0) {
+    delete[] buf;
+    return fd;
+  }
   
   utime_t from = ceph_clock_now(g_ceph_context);
   utime_t start = from;
@@ -2037,7 +2040,10 @@ int SyntheticClient::write_fd(int fd, int size, int wrsize)   // size is in MB,
   uint64_t chunks = (uint64_t)size * (uint64_t)(1024*1024) / (uint64_t)wrsize;
 
   //dout(5) << "SyntheticClient::write_fd: writing to fd " << fd << dendl;
-  if (fd < 0) return fd;
+  if (fd < 0) {
+    delete[] buf;
+    return fd;
+  }
 
   for (unsigned i=0; i<chunks; i++) {
     if (time_to_stop()) {
@@ -2087,7 +2093,10 @@ int SyntheticClient::read_file(const std::string& fn, int size,
 
   int fd = client->open(fn.c_str(), O_RDONLY);
   dout(5) << "reading from " << fn << " fd " << fd << dendl;
-  if (fd < 0) return fd;
+  if (fd < 0) {
+    delete[] buf;
+    return fd;
+  }
 
   utime_t from = ceph_clock_now(g_ceph_context);
   utime_t start = from;
@@ -2694,7 +2703,7 @@ int SyntheticClient::random_walk(int num_req)
     }
 
     // descend?
-    if (.9*roll_die(::pow((double).9,(double)cwd.depth())) && subdirs.size()) {
+    if (.9*roll_die(::pow((double).9,(double)cwd.depth())) && !subdirs.empty()) {
       string s = get_random_subdir();
       cwd.push_dentry( s );
       dout(DBL) << "cd " << s << " -> " << cwd << dendl;
@@ -2824,7 +2833,7 @@ int SyntheticClient::random_walk(int num_req)
       
       for (list<string>::iterator it = c.begin();
            it != c.end();
-           it++) {
+           ++it) {
         //dout(DBL) << " got " << *it << dendl;
 	assert(0);
 	/*contents[*it] = it->second;
