@@ -20,6 +20,8 @@
 #include <time.h>
 
 #include "include/types.h"
+
+
 // --------
 // utime_t
 
@@ -133,6 +135,11 @@ public:
     return ts;
   }
 
+  void sleep() {
+    struct timespec ts = { (__time_t)tv.tv_sec, (long)tv.tv_nsec };
+    nanosleep(&ts, &ts);
+  }
+
   // output
   ostream& gmtime(ostream& out) const {
     out.setf(std::ios::right);
