@@ -31,7 +31,7 @@
 #include "ScatterLock.h"
 #include "LocalLock.h"
 #include "Capability.h"
-#include "snap.h"
+#include "SnapRealm.h"
 
 #include <list>
 #include <vector>
@@ -63,37 +63,6 @@ struct cinode_lock_info_t {
 extern cinode_lock_info_t cinode_lock_info[];
 extern int num_cinode_locks;
 
-/**
- * Default file layout stuff. This lets us set a default file layout on
- * a directory inode that all files in its tree will use on creation.
- */
-struct default_file_layout {
-
-  ceph_file_layout layout;
-
-  default_file_layout() {
-    memset(&layout, 0, sizeof(layout));
-  }
-
-  void encode(bufferlist &bl) const {
-    __u8 struct_v = 1;
-    ::encode(struct_v, bl);
-    ::encode(layout, bl);
-  }
-
-  void decode(bufferlist::iterator& bl) {
-    __u8 struct_v;
-    ::decode(struct_v, bl);
-    if (struct_v != 1) { //uh-oh
-      derr << "got default layout I don't understand!" << dendl;
-      assert(0);
-    }
-    ::decode(layout, bl);
-  }
-};
-WRITE_CLASS_ENCODER(default_file_layout);
-
-
 // cached inode wrapper
 class CInode : public MDSCacheObject {
   /*
@@ -182,7 +151,6 @@ public:
   static const int STATE_NEEDSRECOVER = (1<<11);
   static const int STATE_RECOVERING =   (1<<12);
   static const int STATE_PURGING =     (1<<13);
-  static const int STATE_DIRTYPARENT =  (1<<14);
   static const int STATE_DIRTYRSTAT =  (1<<15);
   static const int STATE_STRAYPINNED = (1<<16);
   static const int STATE_FROZENAUTHPIN = (1<<17);
@@ -221,7 +189,7 @@ public:
     return snaprealm ||  // other snaprealms will link to me
       inode.is_dir() ||  // links to me in other snaps
       inode.nlink > 1 || // there are remote links, possibly snapped, that will need to find me
-      old_inodes.size(); // once multiversion, always multiversion.  until old_inodes gets cleaned out.
+      !old_inodes.empty(); // once multiversion, always multiversion.  until old_inodes gets cleaned out.
   }
   snapid_t get_oldest_snap();
 
@@ -242,8 +210,6 @@ public:
   //bool hack_accessed;
   //utime_t hack_load_stamp;
 
-  default_file_layout *default_layout;
-
   /**
    * Projection methods, used to store inode changes until they have been journaled,
    * at which point they are popped.
@@ -261,14 +227,13 @@ public:
     inode_t *inode;
     map<string,bufferptr> *xattrs;
     sr_t *snapnode;
-    default_file_layout *dir_layout;
 
-    projected_inode_t() : inode(NULL), xattrs(NULL), snapnode(NULL), dir_layout(NULL) {}
-    projected_inode_t(inode_t *in, sr_t *sn) : inode(in), xattrs(NULL), snapnode(sn),
-        dir_layout(NULL) {}
-    projected_inode_t(inode_t *in, map<string, bufferptr> *xp = NULL, sr_t *sn = NULL,
-                      default_file_layout *dl = NULL) :
-      inode(in), xattrs(xp), snapnode(sn), dir_layout(dl) {}
+    projected_inode_t()
+      : inode(NULL), xattrs(NULL), snapnode(NULL) {}
+    projected_inode_t(inode_t *in, sr_t *sn)
+      : inode(in), xattrs(NULL), snapnode(sn) {}
+    projected_inode_t(inode_t *in, map<string, bufferptr> *xp = NULL, sr_t *sn = NULL)
+      : inode(in), xattrs(xp), snapnode(sn) {}
   };
   list<projected_inode_t*> projected_nodes;   // projected values (only defined while dirty)
   
@@ -282,21 +247,6 @@ public:
       return projected_nodes.back();
   }
 
-  ceph_file_layout *get_projected_dir_layout() {
-    if (!inode.is_dir())
-      return NULL;
-    if (projected_nodes.empty()) {
-      if (default_layout)
-        return &default_layout->layout;
-      else
-        return NULL;
-    }
-    else if (projected_nodes.back()->dir_layout)
-      return &projected_nodes.back()->dir_layout->layout;
-    else
-      return NULL;
-  }
-
   version_t get_projected_version() {
     if (projected_nodes.empty())
       return inode.version;
@@ -316,7 +266,7 @@ public:
   inode_t *get_previous_projected_inode() {
     assert(!projected_nodes.empty());
     list<projected_inode_t*>::reverse_iterator p = projected_nodes.rbegin();
-    p++;
+    ++p;
     if (p != projected_nodes.rend())
       return (*p)->inode;
     else
@@ -326,7 +276,7 @@ public:
   map<string,bufferptr> *get_projected_xattrs() {
     for (list<projected_inode_t*>::reverse_iterator p = projected_nodes.rbegin();
 	 p != projected_nodes.rend();
-	 p++)
+	 ++p)
       if ((*p)->xattrs)
 	return (*p)->xattrs;
     return &xattrs;
@@ -335,7 +285,7 @@ public:
     list<projected_inode_t*>::reverse_iterator p = projected_nodes.rbegin();
     for (p++;  // skip the most recent projected value
 	 p != projected_nodes.rend();
-	 p++)
+	 ++p)
       if ((*p)->xattrs)
 	return (*p)->xattrs;
     return &xattrs;
@@ -351,7 +301,7 @@ public:
     } else {
       for (list<projected_inode_t*>::reverse_iterator p = projected_nodes.rbegin();
           p != projected_nodes.rend();
-          p++)
+          ++p)
         if ((*p)->snapnode)
           return (*p)->snapnode;
     }
@@ -393,7 +343,7 @@ public:
   CDir *add_dirfrag(CDir *dir);
   void close_dirfrag(frag_t fg);
   void close_dirfrags();
-  bool has_subtree_root_dirfrag();
+  bool has_subtree_root_dirfrag(int auth=-1);
 
   void force_dirfrags();
   void verify_dirfrags();
@@ -434,7 +384,6 @@ public:
   elist<CInode*>::item item_dirty;
   elist<CInode*>::item item_caps;
   elist<CInode*>::item item_open_file;
-  elist<CInode*>::item item_renamed_file;
   elist<CInode*>::item item_dirty_dirfrag_dir;
   elist<CInode*>::item item_dirty_dirfrag_nest;
   elist<CInode*>::item item_dirty_dirfrag_dirfragtree;
@@ -470,13 +419,12 @@ private:
     snaprealm(0), containing_realm(0),
     first(f), last(l),
     last_journaled(0), //last_open_journaled(0), 
-    default_layout(NULL),
     //hack_accessed(true),
     stickydir_ref(0),
     parent(0),
     inode_auth(CDIR_AUTH_DEFAULT),
     replica_caps_wanted(0),
-    item_dirty(this), item_caps(this), item_open_file(this), item_renamed_file(this), 
+    item_dirty(this), item_caps(this), item_open_file(this),
     item_dirty_dirfrag_dir(this), 
     item_dirty_dirfrag_nest(this), 
     item_dirty_dirfrag_dirfragtree(this), 
@@ -549,7 +497,7 @@ private:
   CInode *get_parent_inode();
   
   bool is_lt(const MDSCacheObject *r) const {
-    CInode *o = (CInode*)r;
+    const CInode *o = static_cast<const CInode*>(r);
     return ino() < o->ino() ||
       (ino() == o->ino() && last < o->last);
   }
@@ -579,52 +527,13 @@ private:
   void fetch(Context *fin);
   void _fetched(bufferlist& bl, bufferlist& bl2, Context *fin);  
 
-  void store_parent(Context *fin);
-  void _stored_parent(version_t v, Context *fin);
+  void fetch_backtrace(inode_backtrace_t *bt, Context *fin);
+  void _fetched_backtrace(bufferlist *bl, inode_backtrace_t *bt, Context *fin);
 
-  void build_backtrace(inode_backtrace_t& bt);
-  unsigned encode_parent_mutation(ObjectOperation& m);
+  void build_backtrace(int64_t location, inode_backtrace_t* bt);
 
-  void encode_store(bufferlist& bl) {
-    __u8 struct_v = 2;
-    ::encode(struct_v, bl);
-    ::encode(inode, bl);
-    if (is_symlink())
-      ::encode(symlink, bl);
-    ::encode(dirfragtree, bl);
-    ::encode(xattrs, bl);
-    bufferlist snapbl;
-    encode_snap_blob(snapbl);
-    ::encode(snapbl, bl);
-    ::encode(old_inodes, bl);
-    if (inode.is_dir()) {
-      ::encode((default_layout ? true : false), bl);
-      if (default_layout)
-        ::encode(*default_layout, bl);
-    }
-  }
-  void decode_store(bufferlist::iterator& bl) {
-    __u8 struct_v;
-    ::decode(struct_v, bl);
-    ::decode(inode, bl);
-    if (is_symlink())
-      ::decode(symlink, bl);
-    ::decode(dirfragtree, bl);
-    ::decode(xattrs, bl);
-    bufferlist snapbl;
-    ::decode(snapbl, bl);
-    decode_snap_blob(snapbl);
-    ::decode(old_inodes, bl);
-    if (struct_v >= 2 && inode.is_dir()) {
-      bool default_layout_exists;
-      ::decode(default_layout_exists, bl);
-      if (default_layout_exists) {
-        delete default_layout;
-        default_layout = new default_file_layout;
-        ::decode(*default_layout, bl);
-      }
-    }
-  }
+  void encode_store(bufferlist& bl);
+  void decode_store(bufferlist::iterator& bl);
 
   void encode_replica(int rep, bufferlist& bl) {
     assert(is_auth());
@@ -638,11 +547,6 @@ private:
     
     _encode_base(bl);
     _encode_locks_state_for_replica(bl);
-    if (inode.is_dir()) {
-      ::encode((default_layout ? true : false), bl);
-      if (default_layout)
-        ::encode(*default_layout, bl);
-    }
   }
   void decode_replica(bufferlist::iterator& p, bool is_new) {
     __u32 nonce;
@@ -651,15 +555,6 @@ private:
     
     _decode_base(p);
     _decode_locks_state(p, is_new);
-    if (inode.is_dir()) {
-      bool default_layout_exists;
-      ::decode(default_layout_exists, p);
-      if (default_layout_exists) {
-        delete default_layout;
-        default_layout = new default_file_layout;
-        ::decode(*default_layout, p);
-      }
-    }
   }
 
 
@@ -788,7 +683,7 @@ public:
     int n = 0;
     for (map<client_t,Capability*>::iterator it = client_caps.begin();
          it != client_caps.end();
-         it++) 
+         ++it) 
       if (!it->second->is_stale())
 	n++;
     return n;
@@ -797,7 +692,7 @@ public:
     int n = 0;
     for (map<client_t,Capability*>::iterator it = client_caps.begin();
          it != client_caps.end();
-         it++) 
+         ++it) 
       if (!it->second->is_stale()) {
 	if (n)
 	  return true;
@@ -841,9 +736,9 @@ public:
 
   // caps issued, wanted
   int get_caps_issued(int *ploner = 0, int *pother = 0, int *pxlocker = 0,
-		      int shift = 0, int mask = 0xffff);
+		      int shift = 0, int mask = -1);
   bool is_any_caps_wanted();
-  int get_caps_wanted(int *ploner = 0, int *pother = 0, int shift = 0, int mask = 0xffff);
+  int get_caps_wanted(int *ploner = 0, int *pother = 0, int shift = 0, int mask = -1);
   bool issued_caps_need_gather(SimpleLock *lock);
   void replicate_relax_locks();
 
@@ -885,22 +780,22 @@ public:
   void bad_put(int by) {
     generic_dout(0) << " bad put " << *this << " by " << by << " " << pin_name(by) << " was " << ref
 #ifdef MDS_REF_SET
-		    << " (" << ref_set << ")"
+		    << " (" << ref_map << ")"
 #endif
 		    << dendl;
 #ifdef MDS_REF_SET
-    assert(ref_set.count(by) == 1);
+    assert(ref_map[by] > 0);
 #endif
     assert(ref > 0);
   }
   void bad_get(int by) {
     generic_dout(0) << " bad get " << *this << " by " << by << " " << pin_name(by) << " was " << ref
 #ifdef MDS_REF_SET
-		    << " (" << ref_set << ")"
+		    << " (" << ref_map << ")"
 #endif
 		    << dendl;
 #ifdef MDS_REF_SET
-    assert(ref_set.count(by) == 0);
+    assert(ref_map[by] >= 0);
 #endif
   }
   void first_get();
