@@ -214,7 +214,7 @@ void PGMap::apply_incremental(CephContext *cct, const Incremental& inc)
   }
   for (set<pg_t>::const_iterator p = inc.pg_remove.begin();
        p != inc.pg_remove.end();
-       p++) {
+       ++p) {
     const pg_t &removed_pg(*p);
     hash_map<pg_t,pg_stat_t>::iterator s = pg_stat.find(removed_pg);
     if (s != pg_stat.end()) {
@@ -225,7 +225,7 @@ void PGMap::apply_incremental(CephContext *cct, const Incremental& inc)
   
   for (set<int>::iterator p = inc.osd_stat_rm.begin();
        p != inc.osd_stat_rm.end();
-       p++) {
+       ++p) {
     hash_map<int,osd_stat_t>::iterator t = osd_stat.find(*p);
     if (t != osd_stat.end()) {
       stat_osd_sub(t->second);
@@ -244,7 +244,7 @@ void PGMap::apply_incremental(CephContext *cct, const Incremental& inc)
   stamp_delta += delta_t;
 
   pg_sum_delta.stats.add(d.stats);
-  if (pg_sum_deltas.size() > (std::list< pair<pool_stat_t, utime_t> >::size_type)MIN(1, cct ? cct->_conf->mon_stat_smooth_intervals : 1)) {
+  if (pg_sum_deltas.size() > (std::list< pair<pool_stat_t, utime_t> >::size_type)MAX(1, cct ? cct->_conf->mon_stat_smooth_intervals : 1)) {
     pg_sum_delta.stats.sub(pg_sum_deltas.front().first.stats);
     stamp_delta -= pg_sum_deltas.front().second;
     pg_sum_deltas.pop_front();
@@ -360,10 +360,11 @@ epoch_t PGMap::calc_min_last_epoch_clean() const
   if (pg_stat.empty())
     return 0;
   hash_map<pg_t,pg_stat_t>::const_iterator p = pg_stat.begin();
-  epoch_t min = p->second.last_epoch_clean;
+  epoch_t min = p->second.get_effective_last_epoch_clean();
   for (++p; p != pg_stat.end(); ++p) {
-    if (p->second.last_epoch_clean < min)
-      min = p->second.last_epoch_clean;
+    epoch_t lec = p->second.get_effective_last_epoch_clean();
+    if (lec < min)
+      min = lec;
   }
   return min;
 }
@@ -537,7 +538,7 @@ void PGMap::dump(ostream& ss) const
   dump_pg_stats_plain(ss, pg_stat);
   for (hash_map<int,pool_stat_t>::const_iterator p = pg_pool_sum.begin();
        p != pg_pool_sum.end();
-       p++)
+       ++p)
     ss << "pool " << p->first
        << "\t" << p->second.stats.sum.num_objects
       //<< "\t" << p->second.num_object_copies
@@ -560,7 +561,7 @@ void PGMap::dump(ostream& ss) const
   ss << "osdstat\tkbused\tkbavail\tkb\thb in\thb out" << std::endl;
   for (hash_map<int,osd_stat_t>::const_iterator p = osd_stat.begin();
        p != osd_stat.end();
-       p++)
+       ++p)
     ss << p->first
        << "\t" << p->second.kb_used
        << "\t" << p->second.kb_avail 
@@ -672,7 +673,6 @@ void PGMap::recovery_summary(ostream& out) const
 	<< si_t(pg_sum_delta.stats.sum.num_bytes_recovered / (double)stamp_delta) << "B/s";
     if (pg_sum_delta.stats.sum.num_keys_recovered)
       out << ", " << si_t(pg_sum_delta.stats.sum.num_keys_recovered / (double)stamp_delta) << " key/s";
-    first = false;
   }
 }
 
