@@ -8,9 +8,7 @@
 #include <string>
 #include <tr1/memory>
 
-#include <string>
 #include <vector>
-#include <tr1/memory>
 
 #include "ObjectMap.h"
 #include "KeyValueDB.h"
@@ -106,7 +104,7 @@ bool DBObjectMap::check(std::ostream &out)
       map<string, bufferlist> got;
       to_get.insert(HEADER_KEY);
       db->get(sys_parent_prefix(header), to_get, &got);
-      if (!got.size()) {
+      if (got.empty()) {
 	out << "Missing: seq " << header.parent << std::endl;
 	retval = false;
 	break;
@@ -244,8 +242,7 @@ bool DBObjectMap::parse_hobject_key_v0(const string &in, coll_t *c,
   *c = coll_t(coll);
   int64_t pool = -1;
   pg_t pg;
-  snapid_t pg_snap;
-  if (c->is_pg(pg, pg_snap))
+  if (c->is_pg_prefix(pg))
     pool = (int64_t)pg.pool();
   (*hoid) = hobject_t(name, key, snap, hash, pool);
   return true;
@@ -556,7 +553,7 @@ int DBObjectMap::_get_header(Header header,
     set<string> to_get;
     to_get.insert(USER_HEADER_KEY);
     int r = db->get(sys_prefix(header), to_get, &out);
-    if (r == 0 && out.size())
+    if (r == 0 && !out.empty())
       break;
     if (r < 0)
       return r;
@@ -566,7 +563,7 @@ int DBObjectMap::_get_header(Header header,
     header = lookup_parent(current);
   }
 
-  if (out.size())
+  if (!out.empty())
     bl->swap(out.begin()->second);
   return 0;
 }
@@ -971,7 +968,7 @@ int DBObjectMap::upgrade()
 		      &got);
       if (r < 0)
 	return r;
-      if (!got.size())
+      if (got.empty())
 	continue; // Moved in a previous transaction
 
       t->rmkeys(USER_PREFIX + header_key(hdr.parent) + SYS_PREFIX,
@@ -1019,7 +1016,7 @@ int DBObjectMap::init(bool do_upgrade)
   int r = db->get(SYS_PREFIX, to_get, &result);
   if (r < 0)
     return r;
-  if (result.size()) {
+  if (!result.empty()) {
     bufferlist::iterator bliter = result.begin()->second.begin();
     state.decode(bliter);
     if (state.v < 1) { // Needs upgrade
@@ -1083,7 +1080,7 @@ DBObjectMap::Header DBObjectMap::_lookup_map_header(const hobject_t &hoid)
   int r = db->get(HOBJECT_TO_SEQ, to_get, &out);
   if (r < 0)
     return Header();
-  if (!out.size())
+  if (out.empty())
     return Header();
   
   Header ret(new _Header(), RemoveMapHeaderOnDelete(this, hoid));
@@ -1126,7 +1123,7 @@ DBObjectMap::Header DBObjectMap::lookup_parent(Header input)
     assert(0);
     return Header();
   }
-  if (out.size() < 1) {
+  if (out.empty()) {
     assert(0);
     return Header();
   }
@@ -1216,3 +1213,16 @@ bool DBObjectMap::check_spos(const hobject_t &hoid,
     return true;
   }
 }
+
+int DBObjectMap::list_objects(vector<hobject_t> *out)
+{
+  KeyValueDB::Iterator iter = db->get_iterator(HOBJECT_TO_SEQ);
+  for (iter->seek_to_first(); iter->valid(); iter->next()) {
+    bufferlist bl = iter->value();
+    bufferlist::iterator bliter = bl.begin();
+    _Header header;
+    header.decode(bliter);
+    out->push_back(header.hoid);
+  }
+  return 0;
+}
