@@ -113,7 +113,7 @@ int HashIndex::col_split_level(
     sub_path.push_back(*i);
     path_to_hobject_hash_prefix(sub_path, &bits, &hash);
     if (bits < inbits) {
-      if ((match & ~((~0)<<bits)) == (hash & ~((~0)<<bits))) {
+      if (hobject_t::match_hash(hash, bits, match)) {
 	r = col_split_level(
 	  from,
 	  to,
@@ -127,7 +127,7 @@ int HashIndex::col_split_level(
 	  *mkdirred = path.size();
       } // else, skip, doesn't need to be moved or recursed into
     } else {
-      if ((match & ~((~0)<<inbits)) == (hash & ~((~0)<<inbits))) {
+      if (hobject_t::match_hash(hash, inbits, match)) {
 	to_move.insert(*i);
       }
     } // else, skip, doesn't need to be moved or recursed into
@@ -138,7 +138,7 @@ int HashIndex::col_split_level(
   for (map<string, hobject_t>::iterator i = objects.begin();
        i != objects.end();
        ++i) {
-    if ((i->second.hash & ~((~0)<<inbits)) == match) {
+    if (i->second.match(inbits, match)) {
       objs_to_move.insert(*i);
     }
   }
@@ -328,6 +328,9 @@ int HashIndex::_collection_list_partial(const hobject_t &start,
 					vector<hobject_t> *ls,
 					hobject_t *next) {
   vector<string> path;
+  hobject_t _next;
+  if (!next)
+    next = &_next;
   *next = start;
   dout(20) << "_collection_list_partial " << start << " " << min_count << "-" << max_count << " ls.size " << ls->size() << dendl;
   return list_by_hash(path, min_count, max_count, seq, next, ls);
@@ -365,21 +368,30 @@ int HashIndex::start_col_split(const vector<string> &path) {
   bufferlist bl;
   InProgressOp op_tag(InProgressOp::COL_SPLIT, path);
   op_tag.encode(bl);
-  return add_attr_path(vector<string>(), IN_PROGRESS_OP_TAG, bl); 
+  int r = add_attr_path(vector<string>(), IN_PROGRESS_OP_TAG, bl);
+  if (r < 0)
+    return r;
+  return fsync_dir(vector<string>());
 }
 
 int HashIndex::start_split(const vector<string> &path) {
   bufferlist bl;
   InProgressOp op_tag(InProgressOp::SPLIT, path);
   op_tag.encode(bl);
-  return add_attr_path(vector<string>(), IN_PROGRESS_OP_TAG, bl); 
+  int r = add_attr_path(vector<string>(), IN_PROGRESS_OP_TAG, bl);
+  if (r < 0)
+    return r;
+  return fsync_dir(vector<string>());
 }
 
 int HashIndex::start_merge(const vector<string> &path) {
   bufferlist bl;
   InProgressOp op_tag(InProgressOp::MERGE, path);
   op_tag.encode(bl);
-  return add_attr_path(vector<string>(), IN_PROGRESS_OP_TAG, bl); 
+  int r = add_attr_path(vector<string>(), IN_PROGRESS_OP_TAG, bl);
+  if (r < 0)
+    return r;
+  return fsync_dir(vector<string>());
 }
 
 int HashIndex::end_split_or_merge(const vector<string> &path) {
@@ -435,18 +447,7 @@ int HashIndex::complete_merge(const vector<string> &path, subdir_info_s info) {
     r = move_objects(path, dst);
     if (r < 0)
       return r;
-    
-    map<string,hobject_t> objects_dst;
-    r = list_objects(dst, 0, 0, &objects_dst);
-    if (r < 0)
-      return r;
-    set<string> subdirs;
-    r = list_subdirs(dst, &subdirs);
-    if (r < 0)
-      return r;
-    dstinfo.objs = objects_dst.size();
-    dstinfo.subdirs = subdirs.size() - 1;
-    r = set_info(dst, dstinfo);
+    r = reset_attr(dst);
     if (r < 0)
       return r;
     r = remove_path(path);
@@ -564,7 +565,7 @@ int HashIndex::complete_split(const vector<string> &path, subdir_info_s info) {
   if (r < 0)
     return r;
   info.objs = objects.size();
-  r = set_info(path, info);
+  r = reset_attr(path);
   if (r < 0)
     return r;
   r = fsync_dir(path);
