@@ -30,6 +30,7 @@ ObjectCacher::BufferHead *ObjectCacher::Object::split(BufferHead *left, loff_t o
   // split off right
   ObjectCacher::BufferHead *right = new BufferHead(this);
   right->last_write_tid = left->last_write_tid;
+  right->last_read_tid = left->last_read_tid;
   right->set_state(left->get_state());
   right->snapc = left->snapc;
 
@@ -62,7 +63,7 @@ ObjectCacher::BufferHead *ObjectCacher::Object::split(BufferHead *left, loff_t o
       ++start_remove;
     for (map<loff_t, list<Context*> >::iterator p = start_remove;
 	 p != left->waitfor_read.end(); ++p) {
-      ldout(oc->cct, 0) << "split  moving waiters at byte " << p->first << " to right bh" << dendl;
+      ldout(oc->cct, 20) << "split  moving waiters at byte " << p->first << " to right bh" << dendl;
       right->waitfor_read[p->first].swap( p->second );
       assert(p->second.empty());
     }
@@ -98,7 +99,7 @@ void ObjectCacher::Object::merge_left(BufferHead *left, BufferHead *right)
   // waiters
   for (map<loff_t, list<Context*> >::iterator p = right->waitfor_read.begin();
        p != right->waitfor_read.end();
-       p++) 
+       ++p) 
     left->waitfor_read[p->first].splice( left->waitfor_read[p->first].begin(),
                                          p->second );
   
@@ -113,22 +114,26 @@ void ObjectCacher::Object::try_merge_bh(BufferHead *bh)
   assert(oc->lock.is_locked());
   ldout(oc->cct, 10) << "try_merge_bh " << *bh << dendl;
 
+  // do not merge rx buffers; last_read_tid may not match
+  if (bh->is_rx())
+    return;
+
   // to the left?
   map<loff_t,BufferHead*>::iterator p = data.find(bh->start());
   assert(p->second == bh);
   if (p != data.begin()) {
-    p--;
+    --p;
     if (p->second->end() == bh->start() &&
 	p->second->get_state() == bh->get_state()) {
       merge_left(p->second, bh);
       bh = p->second;
     } else {
-      p++;
+      ++p;
     }
   }
   // to the right?
   assert(p->second == bh);
-  p++;
+  ++p;
   if (p != data.end() &&
       p->second->start() == bh->end() &&
       p->second->get_state() == bh->get_state())
@@ -151,7 +156,7 @@ bool ObjectCacher::Object::is_cached(loff_t cur, loff_t left)
       loff_t lenfromcur = MIN(p->second->end() - cur, left);
       cur += lenfromcur;
       left -= lenfromcur;
-      p++;
+      ++p;
       continue;
     } else if (p->first > cur) {
       // gap
@@ -176,7 +181,7 @@ int ObjectCacher::Object::map_read(OSDRead *rd,
   assert(oc->lock.is_locked());
   for (vector<ObjectExtent>::iterator ex_it = rd->extents.begin();
        ex_it != rd->extents.end();
-       ex_it++) {
+       ++ex_it) {
     
     if (ex_it->oid != oid.oid)
       continue;
@@ -206,8 +211,7 @@ int ObjectCacher::Object::map_read(OSDRead *rd,
 	  ldout(oc->cct, 20) << "map_read miss " << left << " left, " << *n << dendl;
 	}
         cur += left;
-        left -= left;
-        assert(left == 0);
+        left = 0;
         assert(cur == (loff_t)ex_it->offset + (loff_t)ex_it->length);
         break;  // no more.
       }
@@ -235,7 +239,7 @@ int ObjectCacher::Object::map_read(OSDRead *rd,
         loff_t lenfromcur = MIN(e->end() - cur, left);
         cur += lenfromcur;
         left -= lenfromcur;
-        p++;
+        ++p;
         continue;  // more?
         
       } else if (p->first > cur) {
@@ -311,7 +315,7 @@ ObjectCacher::BufferHead *ObjectCacher::Object::map_write(OSDWrite *wr)
 
   for (vector<ObjectExtent>::iterator ex_it = wr->extents.begin();
        ex_it != wr->extents.end();
-       ex_it++) {
+       ++ex_it) {
 
     if (ex_it->oid != oid.oid) continue;
 
@@ -355,12 +359,12 @@ ObjectCacher::BufferHead *ObjectCacher::Object::map_write(OSDWrite *wr)
           if (cur + max >= p->first + p->second->length()) {
             // we want right bit (one splice)
             final = split(bh, cur);   // just split it, take right half.
-            p++;
+            ++p;
             assert(p->second == final);
           } else {
             // we want middle bit (two splices)
             final = split(bh, cur);
-            p++;
+            ++p;
             assert(p->second == final);
             split(final, cur+max);
           }
@@ -375,7 +379,7 @@ ObjectCacher::BufferHead *ObjectCacher::Object::map_write(OSDWrite *wr)
           if (final) {
 	    oc->mark_dirty(bh);
 	    oc->mark_dirty(final);
-	    p--;  // move iterator back to final
+	    --p;  // move iterator back to final
 	    assert(p->second == final);
             merge_left(final, bh);
 	  } else {
@@ -387,7 +391,7 @@ ObjectCacher::BufferHead *ObjectCacher::Object::map_write(OSDWrite *wr)
         loff_t lenfromcur = final->end() - cur;
         cur += lenfromcur;
         left -= lenfromcur;
-        p++;
+        ++p;
         continue; 
       } else {
         // gap!
@@ -465,7 +469,7 @@ void ObjectCacher::Object::discard(loff_t off, loff_t len)
     // split bh at truncation point?
     if (bh->start() < off) {
       split(bh, off);
-      p++;
+      ++p;
       continue;
     }
 
@@ -474,7 +478,7 @@ void ObjectCacher::Object::discard(loff_t off, loff_t len)
       split(bh, off + len);
     }
 
-    p++;
+    ++p;
     ldout(oc->cct, 10) << "discard " << *this << " bh " << *bh << dendl;
     oc->bh_remove(this, bh);
     delete bh;
@@ -493,22 +497,27 @@ ObjectCacher::ObjectCacher(CephContext *cct_, string name, WritebackHandler& wb,
 			   flush_set_callback_t flush_callback,
 			   void *flush_callback_arg,
 			   uint64_t max_bytes, uint64_t max_objects,
-			   uint64_t max_dirty, uint64_t target_dirty, double max_dirty_age)
+			   uint64_t max_dirty, uint64_t target_dirty,
+			   double max_dirty_age, bool block_writes_upfront)
   : perfcounter(NULL),
     cct(cct_), writeback_handler(wb), name(name), lock(l),
     max_dirty(max_dirty), target_dirty(target_dirty),
     max_size(max_bytes), max_objects(max_objects),
+    block_writes_upfront(block_writes_upfront),
     flush_set_callback(flush_callback), flush_set_callback_arg(flush_callback_arg),
-    flusher_stop(false), flusher_thread(this),
+    last_read_tid(0),
+    flusher_stop(false), flusher_thread(this), finisher(cct),
     stat_clean(0), stat_zero(0), stat_dirty(0), stat_rx(0), stat_tx(0), stat_missing(0),
-    stat_error(0), stat_dirty_waiting(0)
+    stat_error(0), stat_dirty_waiting(0), reads_outstanding(0)
 {
   this->max_dirty_age.set_from_double(max_dirty_age);
   perf_start();
+  finisher.start();
 }
 
 ObjectCacher::~ObjectCacher()
 {
+  finisher.stop();
   perf_stop();
   // we should be empty.
   for (vector<hash_map<sobject_t, Object *> >::iterator i = objects.begin();
@@ -589,13 +598,15 @@ void ObjectCacher::close_object(Object *ob)
 void ObjectCacher::bh_read(BufferHead *bh)
 {
   assert(lock.is_locked());
-  ldout(cct, 7) << "bh_read on " << *bh << dendl;
+  ldout(cct, 7) << "bh_read on " << *bh << " outstanding reads "
+		<< reads_outstanding << dendl;
 
   mark_rx(bh);
+  bh->last_read_tid = ++last_read_tid;
 
   // finisher
-  C_ReadFinish *onfinish = new C_ReadFinish(this, bh->ob->oloc.pool,
-                                            bh->ob->get_soid(), bh->start(), bh->length());
+  C_ReadFinish *onfinish = new C_ReadFinish(this, bh->ob, bh->last_read_tid,
+					    bh->start(), bh->length());
 
   ObjectSet *oset = bh->ob->oset;
 
@@ -604,17 +615,23 @@ void ObjectCacher::bh_read(BufferHead *bh)
 			 bh->start(), bh->length(), bh->ob->get_snap(),
 			 &onfinish->bl, oset->truncate_size, oset->truncate_seq,
 			 onfinish);
+
+  ++reads_outstanding;
 }
 
-void ObjectCacher::bh_read_finish(int64_t poolid, sobject_t oid, loff_t start,
-				  uint64_t length, bufferlist &bl, int r)
+void ObjectCacher::bh_read_finish(int64_t poolid, sobject_t oid, tid_t tid,
+				  loff_t start, uint64_t length,
+				  bufferlist &bl, int r,
+				  bool trust_enoent)
 {
   assert(lock.is_locked());
   ldout(cct, 7) << "bh_read_finish " 
 		<< oid
+		<< " tid " << tid
 		<< " " << start << "~" << length
 		<< " (bl is " << bl.length() << ")"
 		<< " returned " << r
+		<< " outstanding reads " << reads_outstanding
 		<< dendl;
 
   if (bl.length() < length) {
@@ -634,22 +651,54 @@ void ObjectCacher::bh_read_finish(int64_t poolid, sobject_t oid, loff_t start,
     Object *ob = objects[poolid][oid];
     
     if (r == -ENOENT && !ob->complete) {
-      ldout(cct, 7) << "bh_read_finish ENOENT, marking complete and !exists on " << *ob << dendl;
-      ob->complete = true;
-      ob->exists = false;
-
       // wake up *all* rx waiters, or else we risk reordering identical reads. e.g.
       //   read 1~1
       //   reply to unrelated 3~1 -> !exists
       //   read 1~1 -> immediate ENOENT
       //   reply to first 1~1 -> ooo ENOENT
+      bool allzero = true;
       for (map<loff_t, BufferHead*>::iterator p = ob->data.begin(); p != ob->data.end(); ++p) {
 	BufferHead *bh = p->second;
 	for (map<loff_t, list<Context*> >::iterator p = bh->waitfor_read.begin();
 	     p != bh->waitfor_read.end();
-	     p++)
+	     ++p)
 	  ls.splice(ls.end(), p->second);
 	bh->waitfor_read.clear();
+	if (!bh->is_zero() && !bh->is_rx())
+	  allzero = false;
+      }
+
+      // just pass through and retry all waiters if we don't trust
+      // -ENOENT for this read
+      if (trust_enoent) {
+	ldout(cct, 7) << "bh_read_finish ENOENT, marking complete and !exists on " << *ob << dendl;
+	ob->complete = true;
+	ob->exists = false;
+
+	/* If all the bhs are effectively zero, get rid of them.  All
+	 * the waiters will be retried and get -ENOENT immediately, so
+	 * it's safe to clean up the unneeded bh's now. Since we know
+	 * it's safe to remove them now, do so, so they aren't hanging
+	 *around waiting for more -ENOENTs from rados while the cache
+	 * is being shut down.
+	 *
+	 * Only do this when all the bhs are rx or clean, to match the
+	 * condition in _readx(). If there are any non-rx or non-clean
+	 * bhs, _readx() will wait for the final result instead of
+	 * returning -ENOENT immediately.
+	 */
+	if (allzero) {
+	  ldout(cct, 10) << "bh_read_finish ENOENT and allzero, getting rid of "
+			 << "bhs for " << *ob << dendl;
+	  map<loff_t, BufferHead*>::iterator p = ob->data.begin();
+	  while (p != ob->data.end()) {
+	    BufferHead *bh = p->second;
+	    // current iterator will be invalidated by bh_remove()
+	    ++p;
+	    bh_remove(ob, bh);
+	    delete bh;
+	  }
+	}
       }
     }
 
@@ -668,18 +717,18 @@ void ObjectCacher::bh_read_finish(int64_t poolid, sobject_t oid, loff_t start,
 
       BufferHead *bh = p->second;
       ldout(cct, 20) << "checking bh " << *bh << dendl;
-      
+
       // finishers?
       for (map<loff_t, list<Context*> >::iterator it = bh->waitfor_read.begin();
            it != bh->waitfor_read.end();
-           it++)
+           ++it)
 	ls.splice(ls.end(), it->second);
       bh->waitfor_read.clear();
 
       if (bh->start() > opos) {
-        ldout(cct, 1) << "weirdness: gap when applying read results, " 
-                << opos << "~" << bh->start() - opos 
-                << dendl;
+        ldout(cct, 1) << "bh_read_finish skipping gap "
+		      << opos << "~" << bh->start() - opos
+		      << dendl;
         opos = bh->start();
         continue;
       }
@@ -690,6 +739,13 @@ void ObjectCacher::bh_read_finish(int64_t poolid, sobject_t oid, loff_t start,
         continue;
       }
 
+      if (bh->last_read_tid != tid) {
+	ldout(cct, 10) << "bh_read_finish bh->last_read_tid " << bh->last_read_tid
+		       << " != tid " << tid << ", skipping" << dendl;
+	opos = bh->end();
+	continue;
+      }
+
       assert(opos >= bh->start());
       assert(bh->start() == opos);   // we don't merge rx bh's... yet!
       assert(bh->length() <= start+(loff_t)length-opos);
@@ -701,9 +757,14 @@ void ObjectCacher::bh_read_finish(int64_t poolid, sobject_t oid, loff_t start,
       opos = bh->end();
 
       if (r == -ENOENT) {
-	ldout(cct, 10) << "bh_read_finish removing " << *bh << dendl;
-	bh_remove(ob, bh);
-	delete bh;
+	if (trust_enoent) {
+	  ldout(cct, 10) << "bh_read_finish removing " << *bh << dendl;
+	  bh_remove(ob, bh);
+	  delete bh;
+	} else {
+	  ldout(cct, 10) << "skipping unstrusted -ENOENT and will retry for "
+			 << *bh << dendl;
+	}
 	continue;
       }
 
@@ -727,6 +788,8 @@ void ObjectCacher::bh_read_finish(int64_t poolid, sobject_t oid, loff_t start,
   ldout(cct, 20) << "finishing waiters " << ls << dendl;
 
   finish_contexts(cct, ls, err);
+  --reads_outstanding;
+  read_cond.Signal();
 }
 
 
@@ -793,7 +856,7 @@ void ObjectCacher::bh_write_commit(int64_t poolid, sobject_t oid, loff_t start,
     // apply to bh's!
     for (map<loff_t, BufferHead*>::iterator p = ob->data_lower_bound(start);
          p != ob->data.end();
-         p++) {
+         ++p) {
       BufferHead *bh = p->second;
       
       if (bh->start() > start+(loff_t)length)
@@ -835,11 +898,10 @@ void ObjectCacher::bh_write_commit(int64_t poolid, sobject_t oid, loff_t start,
     ob->last_commit_tid = tid;
 
     // waiters?
+    list<Context*> ls;
     if (ob->waitfor_commit.count(tid)) {
-      list<Context*> ls;
       ls.splice(ls.begin(), ob->waitfor_commit[tid]);
       ob->waitfor_commit.erase(tid);
-      finish_contexts(cct, ls, r);
     }
 
     // is the entire object set now clean and fully committed?
@@ -851,6 +913,9 @@ void ObjectCacher::bh_write_commit(int64_t poolid, sobject_t oid, loff_t start,
 	oset->dirty_or_tx == 0) {        // nothing dirty/tx
       flush_set_callback(flush_set_callback_arg, oset);      
     }
+
+    if (!ls.empty())
+      finish_contexts(cct, ls, r);
   }
 }
 
@@ -868,7 +933,7 @@ void ObjectCacher::flush(loff_t amount)
    */
   loff_t did = 0;
   while (amount == 0 || did < amount) {
-    BufferHead *bh = (BufferHead*) bh_lru_dirty.lru_get_next_expire();
+    BufferHead *bh = static_cast<BufferHead*>(bh_lru_dirty.lru_get_next_expire());
     if (!bh) break;
     if (bh->last_write > cutoff) break;
 
@@ -891,7 +956,7 @@ void ObjectCacher::trim(loff_t max_bytes, loff_t max_ob)
 		 << dendl;
 
   while (get_stat_clean() > max_bytes) {
-    BufferHead *bh = (BufferHead*) bh_lru_rest.lru_expire();
+    BufferHead *bh = static_cast<BufferHead*>(bh_lru_rest.lru_expire());
     if (!bh)
       break;
 
@@ -909,7 +974,7 @@ void ObjectCacher::trim(loff_t max_bytes, loff_t max_ob)
   }
 
   while (ob_lru.lru_get_size() > max_ob) {
-    Object *ob = (Object*)ob_lru.lru_expire();
+    Object *ob = static_cast<Object*>(ob_lru.lru_expire());
     if (!ob)
       break;
 
@@ -931,7 +996,7 @@ bool ObjectCacher::is_cached(ObjectSet *oset, vector<ObjectExtent>& extents, sna
   assert(lock.is_locked());
   for (vector<ObjectExtent>::iterator ex_it = extents.begin();
        ex_it != extents.end();
-       ex_it++) {
+       ++ex_it) {
     ldout(cct, 10) << "is_cached " << *ex_it << dendl;
 
     // get Object cache
@@ -969,7 +1034,7 @@ int ObjectCacher::_readx(OSDRead *rd, ObjectSet *oset, Context *onfinish,
 
   for (vector<ObjectExtent>::iterator ex_it = rd->extents.begin();
        ex_it != rd->extents.end();
-       ex_it++) {
+       ++ex_it) {
     ldout(cct, 10) << "readx " << *ex_it << dendl;
 
     total_bytes_read += ex_it->length;
@@ -993,7 +1058,7 @@ int ObjectCacher::_readx(OSDRead *rd, ObjectSet *oset, Context *onfinish,
 	bool wait = false;
 	for (map<loff_t, BufferHead*>::iterator bh_it = o->data.begin();
 	     bh_it != o->data.end();
-	     bh_it++) {
+	     ++bh_it) {
 	  BufferHead *bh = bh_it->second;
 	  if (bh->is_dirty() || bh->is_tx()) {
 	    ldout(cct, 10) << "readx  flushing " << *bh << dendl;
@@ -1014,7 +1079,7 @@ int ObjectCacher::_readx(OSDRead *rd, ObjectSet *oset, Context *onfinish,
       bool allzero = true;
       for (map<loff_t, BufferHead*>::iterator bh_it = o->data.begin();
 	   bh_it != o->data.end();
-	   bh_it++) {
+	   ++bh_it) {
 	ldout(cct, 20) << "readx  ob has bh " << *bh_it->second << dendl;
 	if (!bh_it->second->is_zero() && !bh_it->second->is_rx()) {
 	  allzero = false;
@@ -1045,7 +1110,7 @@ int ObjectCacher::_readx(OSDRead *rd, ObjectSet *oset, Context *onfinish,
       // read missing
       for (map<loff_t, BufferHead*>::iterator bh_it = missing.begin();
            bh_it != missing.end();
-           bh_it++) {
+           ++bh_it) {
         bh_read(bh_it->second);
         if (success && onfinish) {
           ldout(cct, 10) << "readx missed, waiting on " << *bh_it->second 
@@ -1059,7 +1124,7 @@ int ObjectCacher::_readx(OSDRead *rd, ObjectSet *oset, Context *onfinish,
       // bump rx
       for (map<loff_t, BufferHead*>::iterator bh_it = rx.begin();
            bh_it != rx.end();
-           bh_it++) {
+           ++bh_it) {
         touch_bh(bh_it->second);        // bump in lru, so we don't lose it.
         if (success && onfinish) {
           ldout(cct, 10) << "readx missed, waiting on " << *bh_it->second 
@@ -1075,7 +1140,7 @@ int ObjectCacher::_readx(OSDRead *rd, ObjectSet *oset, Context *onfinish,
       // make a plain list
       for (map<loff_t, BufferHead*>::iterator bh_it = hits.begin();
            bh_it != hits.end();
-           bh_it++) {
+           ++bh_it) {
 	ldout(cct, 10) << "readx hit bh " << *bh_it->second << dendl;
 	if (bh_it->second->is_error() && bh_it->second->error)
 	  error = bh_it->second->error;
@@ -1120,11 +1185,11 @@ int ObjectCacher::_readx(OSDRead *rd, ObjectSet *oset, Context *onfinish,
         bhoff += len;
         foff += len;
         if (opos == bh->end()) {
-          bh_it++;
+          ++bh_it;
           bhoff = 0;
         }
         if (foff == f_it->second) {
-          f_it++;
+          ++f_it;
           foff = 0;
         }
         if (bh_it == hits.end()) break;
@@ -1139,7 +1204,7 @@ int ObjectCacher::_readx(OSDRead *rd, ObjectSet *oset, Context *onfinish,
   // bump hits in lru
   for (list<BufferHead*>::iterator bhit = hit_ls.begin();
        bhit != hit_ls.end();
-       bhit++) 
+       ++bhit) 
     touch_bh(*bhit);
   
   if (!success) {
@@ -1172,7 +1237,7 @@ int ObjectCacher::_readx(OSDRead *rd, ObjectSet *oset, Context *onfinish,
     rd->bl->clear();
     for (map<uint64_t,bufferlist>::iterator i = stripe_map.begin();
 	 i != stripe_map.end();
-	 i++) {
+	 ++i) {
       assert(pos == i->first);
       ldout(cct, 10) << "readx  adding buffer len " << i->second.length() << " at " << pos << dendl;
       pos += i->second.length();
@@ -1197,7 +1262,8 @@ int ObjectCacher::_readx(OSDRead *rd, ObjectSet *oset, Context *onfinish,
 }
 
 
-int ObjectCacher::writex(OSDWrite *wr, ObjectSet *oset, Mutex& wait_on_lock)
+int ObjectCacher::writex(OSDWrite *wr, ObjectSet *oset, Mutex& wait_on_lock,
+			 Context *onfreespace)
 {
   assert(lock.is_locked());
   utime_t now = ceph_clock_now(cct);
@@ -1206,7 +1272,7 @@ int ObjectCacher::writex(OSDWrite *wr, ObjectSet *oset, Mutex& wait_on_lock)
   
   for (vector<ObjectExtent>::iterator ex_it = wr->extents.begin();
        ex_it != wr->extents.end();
-       ex_it++) {
+       ++ex_it) {
     // get object cache
     sobject_t soid(ex_it->oid, CEPH_NOSNAP);
     Object *o = get_object(soid, oset, ex_it->oloc);
@@ -1228,7 +1294,7 @@ int ObjectCacher::writex(OSDWrite *wr, ObjectSet *oset, Mutex& wait_on_lock)
     loff_t opos = ex_it->offset;
     for (vector<pair<uint64_t, uint64_t> >::iterator f_it = ex_it->buffer_extents.begin();
          f_it != ex_it->buffer_extents.end();
-         f_it++) {
+         ++f_it) {
       ldout(cct, 10) << "writex writing " << f_it->first << "~" << f_it->second << " into " << *bh << " at " << opos << dendl;
       uint64_t bhoff = bh->start() - opos;
       assert(f_it->second <= bh->length() - bhoff);
@@ -1264,53 +1330,83 @@ int ObjectCacher::writex(OSDWrite *wr, ObjectSet *oset, Mutex& wait_on_lock)
     }
   }
 
-  int r = _wait_for_write(wr, bytes_written, oset, wait_on_lock);
-
+  int r = _wait_for_write(wr, bytes_written, oset, wait_on_lock, onfreespace);
   delete wr;
 
   //verify_stats();
   trim();
   return r;
 }
- 
 
-// blocking wait for write.
-int ObjectCacher::_wait_for_write(OSDWrite *wr, uint64_t len, ObjectSet *oset, Mutex& lock)
+void ObjectCacher::C_WaitForWrite::finish(int r)
+{
+  Mutex::Locker l(m_oc->lock);
+  m_oc->maybe_wait_for_writeback(m_len);
+  m_onfinish->complete(r);
+}
+
+void ObjectCacher::maybe_wait_for_writeback(uint64_t len)
 {
   assert(lock.is_locked());
-  int blocked = 0;
   utime_t start = ceph_clock_now(cct);
+  int blocked = 0;
+  // wait for writeback?
+  //  - wait for dirty and tx bytes (relative to the max_dirty threshold)
+  //  - do not wait for bytes other waiters are waiting on.  this means that
+  //    threads do not wait for each other.  this effectively allows the cache
+  //    size to balloon proportional to the data that is in flight.
+  while (get_stat_dirty() + get_stat_tx() >= max_dirty + get_stat_dirty_waiting()) {
+    ldout(cct, 10) << __func__ << " waiting for dirty|tx "
+		   << (get_stat_dirty() + get_stat_tx()) << " >= max "
+		   << max_dirty << " + dirty_waiting "
+		   << get_stat_dirty_waiting() << dendl;
+    flusher_cond.Signal();
+    stat_dirty_waiting += len;
+    stat_cond.Wait(lock);
+    stat_dirty_waiting -= len;
+    ++blocked;
+    ldout(cct, 10) << __func__ << " woke up" << dendl;
+  }
+  if (blocked && perfcounter) {
+    perfcounter->inc(l_objectcacher_write_ops_blocked);
+    perfcounter->inc(l_objectcacher_write_bytes_blocked, len);
+    utime_t blocked = ceph_clock_now(cct) - start;
+    perfcounter->tinc(l_objectcacher_write_time_blocked, blocked);
+  }
+}
+
+// blocking wait for write.
+int ObjectCacher::_wait_for_write(OSDWrite *wr, uint64_t len, ObjectSet *oset, Mutex& lock, Context *onfreespace)
+{
+  assert(lock.is_locked());
   int ret = 0;
 
   if (max_dirty > 0) {
-    // wait for writeback?
-    //  - wait for dirty and tx bytes (relative to the max_dirty threshold)
-    //  - do not wait for bytes other waiters are waiting on.  this means that
-    //    threads do not wait for each other.  this effectively allows the cache size
-    //    to balloon proportional to the data that is in flight.
-    while (get_stat_dirty() + get_stat_tx() >= max_dirty + get_stat_dirty_waiting()) {
-      ldout(cct, 10) << "wait_for_write waiting on " << len << ", dirty|tx " 
-		     << (get_stat_dirty() + get_stat_tx()) 
-		     << " >= max " << max_dirty << " + dirty_waiting " << get_stat_dirty_waiting()
-		     << dendl;
-      flusher_cond.Signal();
-      stat_dirty_waiting += len;
-      stat_cond.Wait(lock);
-      stat_dirty_waiting -= len;
-      blocked++;
-      ldout(cct, 10) << "wait_for_write woke up" << dendl;
+    if (block_writes_upfront) {
+      maybe_wait_for_writeback(len);
+      if (onfreespace)
+	onfreespace->complete(0);
+    } else {
+      assert(onfreespace);
+      finisher.queue(new C_WaitForWrite(this, len, onfreespace));
     }
   } else {
     // write-thru!  flush what we just wrote.
     Cond cond;
     bool done;
-    C_Cond *fin = new C_Cond(&cond, &done, &ret);
+    Context *fin = block_writes_upfront ?
+      new C_Cond(&cond, &done, &ret) : onfreespace;
+    assert(fin);
     bool flushed = flush_set(oset, wr->extents, fin);
     assert(!flushed);   // we just dirtied it, and didn't drop our lock!
     ldout(cct, 10) << "wait_for_write waiting on write-thru of " << len << " bytes" << dendl;
-    while (!done)
-      cond.Wait(lock);
-    ldout(cct, 10) << "wait_for_write woke up, ret " << ret << dendl;
+    if (block_writes_upfront) {
+      while (!done)
+	cond.Wait(lock);
+      ldout(cct, 10) << "wait_for_write woke up, ret " << ret << dendl;
+      if (onfreespace)
+	onfreespace->complete(ret);
+    }
   }
 
   // start writeback anyway?
@@ -1319,12 +1415,6 @@ int ObjectCacher::_wait_for_write(OSDWrite *wr, uint64_t len, ObjectSet *oset, M
 		   << target_dirty << ", nudging flusher" << dendl;
     flusher_cond.Signal();
   }
-  if (blocked && perfcounter) {
-    perfcounter->inc(l_objectcacher_write_ops_blocked);
-    perfcounter->inc(l_objectcacher_write_bytes_blocked, len);
-    utime_t blocked = ceph_clock_now(cct) - start;
-    perfcounter->tinc(l_objectcacher_write_time_blocked, blocked);
-  }
   return ret;
 }
 
@@ -1357,7 +1447,7 @@ void ObjectCacher::flusher_entry()
       utime_t cutoff = ceph_clock_now(cct);
       cutoff -= max_dirty_age;
       BufferHead *bh = 0;
-      while ((bh = (BufferHead*)bh_lru_dirty.lru_get_next_expire()) != 0 &&
+      while ((bh = static_cast<BufferHead*>(bh_lru_dirty.lru_get_next_expire())) != 0 &&
 	     bh->last_write < cutoff) {
 	ldout(cct, 10) << "flusher flushing aged dirty bh " << *bh << dendl;
 	bh_write(bh);
@@ -1367,6 +1457,19 @@ void ObjectCacher::flusher_entry()
       break;
     flusher_cond.WaitInterval(cct, lock, utime_t(1,0));
   }
+
+  /* Wait for reads to finish. This is only possible if handling
+   * -ENOENT made some read completions finish before their rados read
+   * came back. If we don't wait for them, and destroy the cache, when
+   * the rados reads do come back their callback will try to access the
+   * no-longer-valid ObjectCacher.
+   */
+  while (reads_outstanding > 0) {
+    ldout(cct, 10) << "Waiting for all reads to complete. Number left: "
+		   << reads_outstanding << dendl;
+    read_cond.Wait(lock);
+  }
+
   lock.Unlock();
   ldout(cct, 10) << "flusher finish" << dendl;
 }
@@ -1386,7 +1489,7 @@ bool ObjectCacher::set_is_cached(ObjectSet *oset)
     Object *ob = *p;
     for (map<loff_t,BufferHead*>::iterator q = ob->data.begin();
          q != ob->data.end();
-         q++) {
+         ++q) {
       BufferHead *bh = q->second;
       if (!bh->is_dirty() && !bh->is_tx()) 
         return true;
@@ -1408,7 +1511,7 @@ bool ObjectCacher::set_is_dirty_or_committing(ObjectSet *oset)
     
     for (map<loff_t,BufferHead*>::iterator p = ob->data.begin();
          p != ob->data.end();
-         p++) {
+         ++p) {
       BufferHead *bh = p->second;
       if (bh->is_dirty() || bh->is_tx()) 
         return true;
@@ -1438,7 +1541,7 @@ bool ObjectCacher::flush(Object *ob, loff_t offset, loff_t length)
   assert(lock.is_locked());
   bool clean = true;
   ldout(cct, 10) << "flush " << *ob << " " << offset << "~" << length << dendl;
-  for (map<loff_t,BufferHead*>::iterator p = ob->data_lower_bound(offset); p != ob->data.end(); p++) {
+  for (map<loff_t,BufferHead*>::iterator p = ob->data_lower_bound(offset); p != ob->data.end(); ++p) {
     BufferHead *bh = p->second;
     ldout(cct, 20) << "flush  " << *bh << dendl;
     if (length && bh->start() > offset+length) {
@@ -1457,47 +1560,52 @@ bool ObjectCacher::flush(Object *ob, loff_t offset, loff_t length)
   return clean;
 }
 
+bool ObjectCacher::_flush_set_finish(C_GatherBuilder *gather, Context *onfinish)
+{
+  assert(lock.is_locked());
+  if (gather->has_subs()) {
+    gather->set_finisher(onfinish);
+    gather->activate();
+    return false;
+  }
+
+  ldout(cct, 10) << "flush_set has no dirty|tx bhs" << dendl;
+  onfinish->complete(0);
+  return true;
+}
+
 // flush.  non-blocking, takes callback.
 // returns true if already flushed
 bool ObjectCacher::flush_set(ObjectSet *oset, Context *onfinish)
 {
   assert(lock.is_locked());
+  assert(onfinish != NULL);
   if (oset->objects.empty()) {
     ldout(cct, 10) << "flush_set on " << oset << " dne" << dendl;
-    delete onfinish;
+    onfinish->complete(0);
     return true;
   }
 
   ldout(cct, 10) << "flush_set " << oset << dendl;
 
   // we'll need to wait for all objects to flush!
-  C_GatherBuilder gather(cct, onfinish);
+  C_GatherBuilder gather(cct);
 
-  bool safe = true;
   for (xlist<Object*>::iterator i = oset->objects.begin();
        !i.end(); ++i) {
     Object *ob = *i;
 
     if (!flush(ob, 0, 0)) {
       // we'll need to gather...
-      safe = false;
-
       ldout(cct, 10) << "flush_set " << oset << " will wait for ack tid " 
                << ob->last_write_tid 
                << " on " << *ob
                << dendl;
-      if (onfinish != NULL)
-        ob->waitfor_commit[ob->last_write_tid].push_back(gather.new_sub());
+      ob->waitfor_commit[ob->last_write_tid].push_back(gather.new_sub());
     }
   }
-  if (onfinish != NULL)
-    gather.activate();
-  
-  if (safe) {
-    ldout(cct, 10) << "flush_set " << oset << " has no dirty|tx bhs" << dendl;
-    return true;
-  }
-  return false;
+
+  return _flush_set_finish(&gather, onfinish);
 }
 
 // flush.  non-blocking, takes callback.
@@ -1505,18 +1613,19 @@ bool ObjectCacher::flush_set(ObjectSet *oset, Context *onfinish)
 bool ObjectCacher::flush_set(ObjectSet *oset, vector<ObjectExtent>& exv, Context *onfinish)
 {
   assert(lock.is_locked());
+  assert(onfinish != NULL);
   if (oset->objects.empty()) {
     ldout(cct, 10) << "flush_set on " << oset << " dne" << dendl;
-    delete onfinish;
+    onfinish->complete(0);
     return true;
   }
 
-  ldout(cct, 10) << "flush_set " << oset << " on " << exv.size() << " ObjectExtents" << dendl;
+  ldout(cct, 10) << "flush_set " << oset << " on " << exv.size()
+		 << " ObjectExtents" << dendl;
 
   // we'll need to wait for all objects to flush!
-  C_GatherBuilder gather(cct, onfinish);
+  C_GatherBuilder gather(cct);
 
-  bool safe = true;
   for (vector<ObjectExtent>::iterator p = exv.begin();
        p != exv.end();
        ++p) {
@@ -1530,22 +1639,13 @@ bool ObjectCacher::flush_set(ObjectSet *oset, vector<ObjectExtent>& exv, Context
 
     if (!flush(ob, ex.offset, ex.length)) {
       // we'll need to gather...
-      safe = false;
-
       ldout(cct, 10) << "flush_set " << oset << " will wait for ack tid " 
 		     << ob->last_write_tid << " on " << *ob << dendl;
-      if (onfinish != NULL)
-        ob->waitfor_commit[ob->last_write_tid].push_back(gather.new_sub());
+      ob->waitfor_commit[ob->last_write_tid].push_back(gather.new_sub());
     }
   }
-  if (onfinish != NULL)
-    gather.activate();
-  
-  if (safe) {
-    ldout(cct, 10) << "flush_set " << oset << " has no dirty|tx bhs" << dendl;
-    return true;
-  }
-  return false;
+
+  return _flush_set_finish(&gather, onfinish);
 }
 
 void ObjectCacher::purge_set(ObjectSet *oset)
@@ -1574,7 +1674,7 @@ loff_t ObjectCacher::release(Object *ob)
 
   for (map<loff_t,BufferHead*>::iterator p = ob->data.begin();
        p != ob->data.end();
-       p++) {
+       ++p) {
     BufferHead *bh = p->second;
     if (bh->is_clean() || bh->is_zero())
       clean.push_back(bh);
@@ -1584,7 +1684,7 @@ loff_t ObjectCacher::release(Object *ob)
 
   for (list<BufferHead*>::iterator p = clean.begin();
        p != clean.end();
-       p++) {
+       ++p) {
     bh_remove(ob, *p);
     delete *p;
   }
@@ -1658,7 +1758,7 @@ uint64_t ObjectCacher::release_all()
     hash_map<sobject_t, Object*>::iterator p = i->begin();
     while (p != i->end()) {
       hash_map<sobject_t, Object*>::iterator n = p;
-      n++;
+      ++n;
 
       Object *ob = p->second;
 
@@ -1681,7 +1781,26 @@ uint64_t ObjectCacher::release_all()
   return unclean;
 }
 
+void ObjectCacher::clear_nonexistence(ObjectSet *oset)
+{
+  assert(lock.is_locked());
+  ldout(cct, 10) << "clear_nonexistence() " << oset << dendl;
 
+  for (xlist<Object*>::iterator p = oset->objects.begin();
+       !p.end(); ++p) {
+    Object *ob = *p;
+    if (!ob->exists) {
+      ldout(cct, 10) << " setting exists and complete on " << *ob << dendl;
+      ob->exists = true;
+      ob->complete = false;
+    }
+    for (xlist<C_ReadFinish*>::iterator q = ob->reads.begin();
+	 !q.end(); ++q) {
+      C_ReadFinish *comp = *q;
+      comp->distrust_enoent();
+    }
+  }
+}
 
 /**
  * discard object extents from an ObjectSet by removing the objects in exls from the in-memory oset.
@@ -1732,7 +1851,7 @@ void ObjectCacher::verify_stats() const
       Object *ob = p->second;
       for (map<loff_t, BufferHead*>::const_iterator q = ob->data.begin();
           q != ob->data.end();
-          q++) {
+          ++q) {
         BufferHead *bh = q->second;
         switch (bh->get_state()) {
         case BufferHead::STATE_MISSING:
