@@ -10,6 +10,7 @@
 #include "rgw_acl.h"
 #include "rgw_policy_s3.h"
 #include "rgw_user.h"
+#include "rgw_cors.h"
 
 #include "common/armor.h"
 
@@ -71,7 +72,6 @@ int RGWGetObj_ObjStore_S3::send_response_data(bufferlist& bl, off_t bl_ofs, off_
 {
   const char *content_type = NULL;
   string content_type_str;
-  int req_state = ret;
   map<string, string> response_attrs;
   map<string, string>::iterator riter;
 
@@ -117,8 +117,9 @@ int RGWGetObj_ObjStore_S3::send_response_data(bufferlist& bl, off_t bl_ofs, off_
 	if (response_attrs.count(aiter->second) > 0) // was already overridden by a response param
 	  continue;
 
-	if ((!content_type) && aiter->first.compare(RGW_ATTR_CONTENT_TYPE) == 0) { // special handling for content_type
-	  content_type = iter->second.c_str();
+	if (aiter->first.compare(RGW_ATTR_CONTENT_TYPE) == 0) { // special handling for content_type
+	  if (!content_type)
+	    content_type = iter->second.c_str();
 	  continue;
         }
 	response_attrs[aiter->second] = iter->second.c_str();
@@ -131,10 +132,8 @@ int RGWGetObj_ObjStore_S3::send_response_data(bufferlist& bl, off_t bl_ofs, off_
     }
   }
 
-  if (partial_content && !ret)
-    req_state = STATUS_PARTIAL_CONTENT;
 done:
-  set_req_state_err(s, req_state);
+  set_req_state_err(s, (partial_content && !ret) ? STATUS_PARTIAL_CONTENT : ret);
 
   dump_errno(s);
 
@@ -157,29 +156,44 @@ send_data:
   return 0;
 }
 
-void RGWListBuckets_ObjStore_S3::send_response()
+void RGWListBuckets_ObjStore_S3::send_response_begin(bool has_buckets)
 {
   if (ret)
     set_req_state_err(s, ret);
   dump_errno(s);
   dump_start(s);
+  end_header(s, "application/xml");
 
-  list_all_buckets_start(s);
-  dump_owner(s, s->user.user_id, s->user.display_name);
+  if (!ret) {
+    list_all_buckets_start(s);
+    dump_owner(s, s->user.user_id, s->user.display_name);
+    s->formatter->open_array_section("Buckets");
+    sent_data = true;
+  }
+}
+
+void RGWListBuckets_ObjStore_S3::send_response_data(RGWUserBuckets& buckets)
+{
+  if (!sent_data)
+    return;
 
   map<string, RGWBucketEnt>& m = buckets.get_buckets();
   map<string, RGWBucketEnt>::iterator iter;
 
-  s->formatter->open_array_section("Buckets");
   for (iter = m.begin(); iter != m.end(); ++iter) {
     RGWBucketEnt obj = iter->second;
     dump_bucket(s, obj);
   }
-  s->formatter->close_section();
-  list_all_buckets_end(s);
-  dump_content_length(s, s->formatter->get_len());
-  end_header(s, "application/xml");
-  rgw_flush_formatter_and_reset(s, s->formatter);
+  rgw_flush_formatter(s, s->formatter);
+}
+
+void RGWListBuckets_ObjStore_S3::send_response_end()
+{
+  if (sent_data) {
+    s->formatter->close_section();
+    list_all_buckets_end(s);
+    rgw_flush_formatter_and_reset(s, s->formatter);
+  }
 }
 
 int RGWListBucket_ObjStore_S3::get_params()
@@ -685,13 +699,6 @@ void RGWPostObj_ObjStore_S3::rebuild_key(string& key)
 
 int RGWPostObj_ObjStore_S3::get_params()
 {
-  string temp_line;
-  string param;
-  string old_param;
-  string param_value;
-
-  string whitespaces (" \t\f\v\n\r");
-
   // get the part boundary
   string req_content_type_str = s->env->get("CONTENT_TYPE", "");
   string req_content_type;
@@ -790,6 +797,11 @@ int RGWPostObj_ObjStore_S3::get_params()
 
   rebuild_key(s->object_str);
 
+  if (s->object_str.empty()) {
+    err_msg = "Empty object name";
+    return -EINVAL;
+  }
+
   env.add_var("key", s->object_str);
 
   part_str("Content-Type", &content_type);
@@ -827,7 +839,6 @@ int RGWPostObj_ObjStore_S3::get_params()
 int RGWPostObj_ObjStore_S3::get_policy()
 {
   bufferlist encoded_policy;
-  string uid;
 
   if (part_bl("policy", &encoded_policy)) {
 
@@ -1238,6 +1249,66 @@ void RGWPutACLs_ObjStore_S3::send_response()
   dump_start(s);
 }
 
+void RGWGetCORS_ObjStore_S3::send_response()
+{
+  if (ret) {
+    if (ret == -ENOENT) 
+      set_req_state_err(s, ERR_NOT_FOUND);
+    else 
+      set_req_state_err(s, ret);
+  }
+  dump_errno(s);
+  end_header(s, "application/xml");
+  dump_start(s);
+  if (!ret) {
+    s->cio->write(cors.c_str(), cors.size());
+  }
+}
+
+void RGWPutCORS_ObjStore_S3::send_response()
+{
+  if (ret)
+    set_req_state_err(s, ret);
+  dump_errno(s);
+  end_header(s, "application/xml");
+  dump_start(s);
+}
+
+void RGWDeleteCORS_ObjStore_S3::send_response()
+{
+  int r = ret;
+  if (!r || r == -ENOENT)
+    r = STATUS_NO_CONTENT;
+
+  set_req_state_err(s, r);
+  dump_errno(s);
+  end_header(s);
+}
+
+void RGWOptionsCORS_ObjStore_S3::send_response()
+{
+  string hdrs, exp_hdrs;
+  uint32_t max_age = CORS_MAX_AGE_INVALID;
+  /*EACCES means, there is no CORS registered yet for the bucket
+   *ENOENT means, there is no match of the Origin in the list of CORSRule
+   *ENOTSUPP means, the HTTP_METHOD is not supported
+   */
+  if (ret == -ENOENT)
+    ret = -EACCES;
+  if (ret != -EACCES) {
+    get_response_params(hdrs, exp_hdrs, &max_age);
+  } else {
+    set_req_state_err(s, ret);
+    dump_errno(s);
+    end_header(s);
+    return;
+  }
+
+  dump_errno(s);
+  dump_access_control(s, origin, req_meth, hdrs.c_str(), exp_hdrs.c_str(), max_age); 
+  end_header(s);
+}
+
 int RGWInitMultipart_ObjStore_S3::get_params()
 {
   RGWAccessControlPolicy_S3 s3policy(s->cct);
@@ -1443,7 +1514,7 @@ void RGWDeleteMultiObj_ObjStore_S3::send_partial_response(pair<string,int>& resu
       s->formatter->open_object_section("Deleted");
       s->formatter->dump_string("Key", result.first);
       s->formatter->close_section();
-    } else if (result.first < 0) {
+    } else if (result.second < 0) {
       struct rgw_html_errors r;
       int err_no;
 
@@ -1493,6 +1564,8 @@ RGWOp *RGWHandler_ObjStore_Bucket_S3::op_get()
     return new RGWGetBucketLogging_ObjStore_S3;
   if (is_acl_op()) {
     return new RGWGetACLs_ObjStore_S3;
+  } else if (is_cors_op()) {
+    return new RGWGetCORS_ObjStore_S3;
   } else if (s->args.exists("uploads")) {
     return new RGWListBucketMultiparts_ObjStore_S3;
   }
@@ -1515,12 +1588,17 @@ RGWOp *RGWHandler_ObjStore_Bucket_S3::op_put()
     return NULL;
   if (is_acl_op()) {
     return new RGWPutACLs_ObjStore_S3;
-  }
+  } else if (is_cors_op()) {
+    return new RGWPutCORS_ObjStore_S3;
+  } 
   return new RGWCreateBucket_ObjStore_S3;
 }
 
 RGWOp *RGWHandler_ObjStore_Bucket_S3::op_delete()
 {
+  if (is_cors_op()) {
+    return new RGWDeleteCORS_ObjStore_S3;
+  }
   return new RGWDeleteBucket_ObjStore_S3;
 }
 
@@ -1533,6 +1611,11 @@ RGWOp *RGWHandler_ObjStore_Bucket_S3::op_post()
   return new RGWPostObj_ObjStore_S3;
 }
 
+RGWOp *RGWHandler_ObjStore_Bucket_S3::op_options()
+{
+  return new RGWOptionsCORS_ObjStore_S3;
+}
+
 RGWOp *RGWHandler_ObjStore_Obj_S3::get_obj_op(bool get_data)
 {
   if (is_acl_op()) {
@@ -1595,6 +1678,11 @@ RGWOp *RGWHandler_ObjStore_Obj_S3::op_post()
   return NULL;
 }
 
+RGWOp *RGWHandler_ObjStore_Obj_S3::op_options()
+{
+  return new RGWOptionsCORS_ObjStore_S3;
+}
+
 int RGWHandler_ObjStore_S3::init_from_header(struct req_state *s, int default_formatter, bool configurable_format)
 {
   string req;
@@ -1675,7 +1763,7 @@ static bool looks_like_ip_address(const char *bucket)
   return (num_periods == 3);
 }
 
-int RGWHandler_ObjStore_S3::validate_bucket_name(const string& bucket)
+int RGWHandler_ObjStore_S3::validate_bucket_name(const string& bucket, bool relaxed_names)
 {
   int ret = RGWHandler_ObjStore::validate_bucket_name(bucket);
   if (ret < 0)
@@ -1684,10 +1772,13 @@ int RGWHandler_ObjStore_S3::validate_bucket_name(const string& bucket)
   if (bucket.size() == 0)
     return 0;
 
+  // bucket names must start with a number, letter, or underscore
   if (!(isalpha(bucket[0]) || isdigit(bucket[0]))) {
-    // bucket names must start with a number or letter
-    return -ERR_INVALID_BUCKET_NAME;
-  }
+    if (!relaxed_names)
+      return -ERR_INVALID_BUCKET_NAME;
+    else if (!(bucket[0] == '_' || bucket[0] == '.' || bucket[0] == '-'))
+      return -ERR_INVALID_BUCKET_NAME;
+  } 
 
   for (const char *s = bucket.c_str(); *s; ++s) {
     char c = *s;
@@ -1711,7 +1802,8 @@ int RGWHandler_ObjStore_S3::init(RGWRados *store, struct req_state *s, RGWClient
 {
   dout(10) << "s->object=" << (s->object ? s->object : "<NULL>") << " s->bucket=" << (s->bucket_name ? s->bucket_name : "<NULL>") << dendl;
 
-  int ret = validate_bucket_name(s->bucket_name_str);
+  bool relaxed_names = s->cct->_conf->rgw_relaxed_s3_bucket_names;
+  int ret = validate_bucket_name(s->bucket_name_str, relaxed_names);
   if (ret)
     return ret;
   ret = validate_object_name(s->object_str);
@@ -1731,6 +1823,16 @@ int RGWHandler_ObjStore_S3::init(RGWRados *store, struct req_state *s, RGWClient
   return RGWHandler_ObjStore::init(store, s, cio);
 }
 
+// subresources that we shouldn't include in the uri to sign
+static const char *nonsigned_subresources[] = {
+  "key",
+  "subuser",
+  "caps",
+  "index",
+  "object",
+  NULL
+};
+
 /*
  * ?get the canonical amazon-style header for something?
  */
@@ -1840,6 +1942,22 @@ static bool get_auth_header(struct req_state *s, string& dest, bool qsr)
 
   string canon_resource;
   get_canon_resource(s, canon_resource);
+
+  map<string, string> sub_resources = s->args.get_sub_resources();
+  map<string, string>::iterator sres_iter;
+
+  for (const char **sres = nonsigned_subresources; *sres; ++sres) {
+    sres_iter = sub_resources.find(*sres);
+    if (sres_iter != sub_resources.end()) {
+      size_t pos = canon_resource.find(*sres) -1;
+      size_t len = strlen(*sres) + 1;
+      std::string sres_val = sres_iter->second;
+      if (!sres_val.empty())
+        len += sres_val.size();
+      canon_resource.erase(pos, len);
+    }
+  }
+
   dest.append(canon_resource);
 
   return true;
